// SPDX-License-Identifier: GPL-3.0
pragma solidity ^0.8.23;

import {Ownable} from "@openzeppelin/contracts/access/Ownable.sol";
import {DCO2s} from "./DCO2s.sol";

contract AA is Ownable {
    DCO2s public dco2sc;
    string[] nfts;

    mapping(address => bool) public airdropped;
    mapping(address => bool) public eligible;

    constructor(string memory name, string memory symbol, string memory baseURI, string[] memory nftURIs)
        Ownable(msg.sender)
    {
        dco2sc = new DCO2s(address(this), address(this), baseURI, name, symbol);
        nfts = nftURIs;
    }

    // Auction Logic
    // modifier to start and finish auction
    // ToDo Check blockchain bid example
    // function bid and lock tokens
    // function cashback

    // Mint Function
    // Simplify
    function mintNFT(address to, uint256 index) external onlyOwner {
        require(!airdropped[to], "account already airdropped");
        dco2sc.safeMint(to, nfts[index]);
        removeNFTAt(index);
        airdropped[to] = true;
    }

    // @Notice Check if eligible
    function checkIfEligible() public view returns (bool winner) {
        if (airdropped[msg.sender] == true) {
            winner = false;
        } else if (eligible[msg.sender] == true) {
            winner = true;
        } else {
            winner = false;
        }
        // require( callable until specific date)
        // Activity
        // Longest Holders
        // Top Holders
        // or Allowed Wallets()
        return winner;
    }

    // @notice Random giveaway function
    function airdrop() external {
        require(!airdropped[msg.sender], "account already airdropped");
        require(checkIfEligible(), "account not eligible");
        // top 20 is for Auction
        uint256 randomURI = getRandomNumber() % nfts.length - 20;
        dco2sc.safeMint(msg.sender, nfts[randomURI]);
        removeNFTAt(randomURI);
        // Require remove the URI so that is not mintable again
        airdropped[msg.sender] = true;
    }

    // @notice adds addresses to the eligible list
    // thanks for supporting DECA
    function inviteEligible(address supporter) public onlyOwner {
        require(!airdropped[msg.sender], "account already airdropped");
        eligible[supporter] = true;
    }

    // @notice Returns a randon number calculated from previews block randao
    // @dev This only works after the merge
    function getRandomNumber() public view returns (uint256 randomNumber) {
        randomNumber = block.prevrandao;
    }

    // @notice removes NFT URI from index
    function removeNFTAt(uint256 index) public {
        require(index < nfts.length, "nft index out of bounds");
        nfts[index] = nfts[nfts.length - 1];
        nfts.pop();
    }
}
